﻿'
' Will Strohl (will.strohl@gmail.com)
' http://www.willstrohl.com
'
'Copyright (c) 2012, Will Strohl
'All rights reserved.
'
'Redistribution and use in source and binary forms, with or without modification, are 
'permitted provided that the following conditions are met:
'
'Redistributions of source code must retain the above copyright notice, this list of 
'conditions and the following disclaimer.
'
'Redistributions in binary form must reproduce the above copyright notice, this list 
'of conditions and the following disclaimer in the documentation and/or other 
'materials provided with the distribution.
'
'Neither the name of Will Strohl, Simple Redirect, nor the names of its contributors may be 
'used to endorse or promote products derived from this software without specific prior 
'written permission.
'
'THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
'EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
'OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
'SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
'INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
'TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
'BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
'CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
'ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
'DAMAGE.
'

'Imports DotNetNuke.Entities.Modules

NameSpace WillStrohl.Modules.SimpleRedirect

    Public Class RedirectController
        'Implements IPortable

#Region " Private Members "

        Private Const CACHEKEY_GETREDIRECTS As String = "WillStrohl-SimpleRedirect-GetRedirects"
        Private Const CACHEKEY_GETREDIRECTSREVERSE As String = "WillStrohl-SimpleRedirect-GetRedirectsReverse"

        Public Const PARAM_REDIRECTID As String = "rid"
        Public Const PARAM_ACTION As String = "action"

        Public Const SETTING_TRACKING As String = "Tracking"
        Public Const SETTING_DEBUG As String = "Debug"
        Public Const SETTING_ENABLE As String = "Enable"

        Public Const INVALID_REQUEST_PATCH_MATCH_PATTERN As String = "(ctl=Edit|mid=\d+|popUp=true|ctl/Edit|mid/\d+|/Admin/|/Host/|/DesktopModules/)"

#End Region

#Region " Data Access "

        Public Function AddRedirect(ModuleId As Integer, RedirectFrom As String, RedirectTo As String) As Integer
            Return DataProvider.Instance().AddRedirect(ModuleId, RedirectFrom, RedirectTo)
        End Function

        Public Function GetRedirect(RedirectId As Integer) As RedirectInfo
            Dim oRedirect As New RedirectInfo
            oRedirect.Fill(DataProvider.Instance().GetRedirect(RedirectId))
            Return oRedirect
        End Function

        Public Function GetRedirectByUrl(RedirectTo As String) As RedirectInfo
            Dim oRedirect As New RedirectInfo
            oRedirect.Fill(DataProvider.Instance().GetRedirectByUrl(RedirectTo))
            Return oRedirect
        End Function

        Public Function GetRedirects(ModuleId As Integer) As RedirectInfoCollection
            Return GetRedirects(ModuleId, False)
        End Function

        Public Function GetRedirects(ModuleId As Integer, ByVal IgnoreCache As Boolean) As RedirectInfoCollection

            Dim collRedirect As New RedirectInfoCollection

            If IgnoreCache Then
                collRedirect.Fill(DataProvider.Instance().GetRedirects(ModuleId))
            Else
                If Not DataCache.GetCache(CACHEKEY_GETREDIRECTS) Is Nothing Then
                    collRedirect = CType(DataCache.GetCache(CACHEKEY_GETREDIRECTS), RedirectInfoCollection)
                Else
                    collRedirect.Fill(DataProvider.Instance().GetRedirects(ModuleId))
                    DataCache.SetCache(CACHEKEY_GETREDIRECTS, collRedirect, DateTime.Now.AddHours(1))
                End If
            End If

            Return collRedirect

        End Function

        Public Function GetRedirectsReverse(ModuleId As Integer, ByVal IgnoreCache As Boolean) As RedirectInfoCollection

            Dim collRedirect As New RedirectInfoCollection

            If IgnoreCache Then
                collRedirect.Fill(DataProvider.Instance().GetRedirectsReverse(ModuleId))
            Else
                If Not DataCache.GetCache(CACHEKEY_GETREDIRECTSREVERSE) Is Nothing Then
                    collRedirect = CType(DataCache.GetCache(CACHEKEY_GETREDIRECTSREVERSE), RedirectInfoCollection)
                Else
                    collRedirect.Fill(DataProvider.Instance().GetRedirectsReverse(ModuleId))
                    DataCache.SetCache(CACHEKEY_GETREDIRECTSREVERSE, collRedirect, DateTime.Now.AddHours(1))
                End If
            End If

            Return collRedirect

        End Function

        Public Sub UpdateRedirect(RedirectId As Integer, ModuleId As Integer, RedirectFrom As String, RedirectTo As String)
            DataProvider.Instance().UpdateRedirect(RedirectId, ModuleId, RedirectFrom, RedirectTo)
        End Sub

        Public Sub DeleteRedirect(RedirectId As Integer)
            DataProvider.Instance().DeleteRedirect(RedirectId)
        End Sub

        Public Function AddVisitor(ByVal RedirectId As Integer, ByVal ReferringUrl As String, ByVal ReferringDomain As String) As Integer
            Return DataProvider.Instance().AddVisitor(RedirectId, ReferringUrl, ReferringDomain)
        End Function

        Public Sub DeleteVisitorsByRedirect(ByVal RedirectId As Integer)
            DataProvider.Instance().DeleteVisitorsByRedirect(RedirectId)
        End Sub

        Public Sub DeleteVisitorsByModule(ByVal ModuleId As Integer)
            DataProvider.Instance().DeleteVisitorsByModule(ModuleId)
        End Sub

        Public Sub ClearVisitorLog()
            DataProvider.Instance().ClearVisitorLog()
        End Sub

        Public Function GetDomainsByRedirect(ByVal RedirectId As Integer) As ArrayList
            Dim oDomains As New ArrayList()
            Dim dr As IDataReader = DataProvider.Instance().GetDomainsByRedirect(RedirectId)

            While dr.Read()
                oDomains.Add(Convert.ToString(dr(0)))
            End While
            dr.Close()

            Return oDomains
        End Function

        Public Function GetRedirectVisitCount(ByVal RedirectId As Integer) As Integer
            Return DataProvider.Instance().GetRedirectVisitCount(RedirectId)
        End Function

        Public Function GetUrlsByRedirect(ByVal RedirectId As Integer) As ArrayList
            Dim oUrls As New ArrayList()
            Dim dr As IDataReader = DataProvider.Instance().GetUrlsByRedirect(RedirectId)

            While dr.Read()
                oUrls.Add(Convert.ToString(dr(0)))
            End While
            dr.Close()

            Return oUrls
        End Function

#End Region

    End Class

End Namespace